@@ -48,19 +48,7 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
     configFileRead(false),
     helpLang("en")
 {
-/*
-  #if defined (_WIN32)
-   char pathAndName[MAX_PATH], path[MAX_PATH], drive[3];
-   ::GetModuleFileName(NULL, pathAndName, MAX_PATH);
-    _splitpath(pathAndName, drive, path, 0, 0);
-    appPath = std::string(drive)+path;
 
-  appPath= string(argv[0]);
-  string::size_type Pos = appPath.find_last_of('\\');
-  appPath = appPath.substr(0, Pos+1);
-
-  #endif
-*/
   loadConfigurationFile();
 
   int c, option_index = 0;
@@ -84,6 +72,7 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         {OPT_LATEX,0, 0,S_OPT_LATEX},
         {OPT_XSLFO,0, 0,S_OPT_XSLFO},
         {OPT_ANSI,0, 0,S_OPT_ANSI},
+        {OPT_XML,0, 0,S_OPT_XML},
         {OPT_BATCHREC,1,0,S_OPT_BATCHREC},
         {OPT_FRAGMENT,0,0,S_OPT_FRAGMENT},
         {OPT_ANCHORS, 0,0,S_OPT_ANCHORS },
@@ -181,6 +170,9 @@ CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
         case S_OPT_ANSI:
           outputType=highlight::ANSI;
           break;
+        case S_OPT_XML:
+          outputType=highlight::XML;
+          break;
         case S_OPT_BATCHREC:
           opt_batch_mode = true;
           readDirectory(string(optarg));
@@ -523,7 +515,6 @@ void CmdLineOptions::loadConfigurationFile()
         {
           outputType=highlight::ANSI;
         }
-
     }
 }
 
@@ -533,5 +524,5 @@ string CmdLineOptions::validateDirPath(const string & path){
 }
 
 highlight::OutputType CmdLineOptions::getOutputType() const {
-  return outputType;
+    return outputType;
 }
