@@ -83,6 +83,7 @@
 #define OPT_PROGRESSBAR  "progress"
 #define OPT_FILLZEROES   "zeroes"
 #define OPT_ANSI         "ansi"
+#define OPT_XML          "xml"
 
 #define S_OPT_ANSI       'A'
 #define S_OPT_OUT        'o'
@@ -102,6 +103,7 @@
 #define S_OPT_TEX        'T'
 #define S_OPT_LATEX      'L'
 #define S_OPT_XSLFO      'Y'
+#define S_OPT_XML        'Z'
 #define S_OPT_BATCHREC   'B'
 #define S_OPT_FRAGMENT   'f'
 #define S_OPT_ANCHORS    'a'
@@ -122,7 +124,7 @@
 #define S_OPT_PROGRESSBAR    'P'
 #define S_OPT_FILLZEROES     'z'
 
-#define S_OPTIONS_STRING "o:i:S:B:O:s:c:e:t:F:D:H:E:afghlvwpqrzACILYRTXVWP"
+#define S_OPTIONS_STRING "o:i:S:B:O:s:c:e:t:F:D:H:E:afghlvwpqrzACILYRTZXV::W::P"
 
 using namespace std;
 
