@@ -23,6 +23,7 @@
 #include "latexgenerator.h"
 #include "texgenerator.h"
 #include "xslfogenerator.h"
+#include "xmlgenerator.h"
 #ifndef USE_WX_LIBRARY
   #include "ansigenerator.h"
 #endif
@@ -62,6 +63,9 @@ CodeGenerator* CodeGenerator::getInstance(OutputType type,
       case XSLFO:
         generator = new XslFoGenerator(styleInfoPath, fopCompatible);
         break;
+      case XML:
+        generator = new XmlGenerator(styleInfoPath);
+        break;
       case XHTML:
         generator = new XHtmlGenerator(styleInfoPath,
                                        styleInPath,
@@ -83,12 +87,13 @@ CodeGenerator* CodeGenerator::getInstance(OutputType type,
      }
   }
 
-  generator->setNumberSpaces(numSpaces);
   generator->setPrintLineNumbers(ln);
   generator->setPrintZeroes(lnz);
   generator->setFragmentCode(fragment);
-  generator->setLineWrapping(lineWrappingMode,
-             (generator->getPrintLineNumbers())?80-LINE_NUMBER_WIDTH:80);
+  generator->setPreformatting(lineWrappingMode,
+                             (generator->getPrintLineNumbers()) ?
+                                MAX_LINE__WIDTH - LINE_NUMBER_WIDTH : MAX_LINE__WIDTH,
+                              numSpaces );
   return generator;
 }
 
@@ -107,10 +112,9 @@ CodeGenerator::CodeGenerator():
     showLineNumbers (false),
     lineNumberFillZeroes(false),
     lineNumber(0),
-    numberSpaces(0),
     lineIndex(0),
     formatter(NULL),
-    lineWrapper(NULL),
+    preFormatter(NULL),
     formattingEnabled(false),
     formattingPossible(false)
 {}
@@ -125,10 +129,9 @@ CodeGenerator::CodeGenerator(const string &colourTheme)
     lineNumberFillZeroes(false),
     lineNumber(0),
     stylePath(colourTheme),
-    numberSpaces(0),
     lineIndex(0),
     formatter(NULL),
-    lineWrapper(NULL),
+    preFormatter(NULL),
     formattingEnabled(false),
     formattingPossible(false)
 {
@@ -138,7 +141,7 @@ CodeGenerator::CodeGenerator(const string &colourTheme)
 
 CodeGenerator::~CodeGenerator()
 {
-  delete lineWrapper;
+  delete preFormatter;
   delete formatter;
 }
 
@@ -161,7 +164,6 @@ bool CodeGenerator::getPrintZeroes(){
   return lineNumberFillZeroes;
 }
 
-
 void CodeGenerator::setFragmentCode(bool flag){
   fragmentOutput=flag;
 }
@@ -170,14 +172,6 @@ bool CodeGenerator::getFragmentCode(){
   return fragmentOutput;
 }
 
-void CodeGenerator::setNumberSpaces(int num){
-  numberSpaces=num;
-}
-
-int CodeGenerator::getNumberSpaces(){
-  return numberSpaces;
-}
-
 void CodeGenerator::setStyleName(const string& s){
   stylePath=s;
 }
@@ -194,18 +188,26 @@ bool CodeGenerator::formattingIsPossible(){
   return formattingPossible;
 }
 
-void CodeGenerator::setLineWrapping(WrapMode lineWrappingStyle,
-                                    unsigned int lineLength){
-  if (lineWrappingStyle!=WRAP_DISABLED)  {
-    lineWrapper=new LineWrapper(lineLength, lineWrappingStyle==WRAP_DEFAULT);
+void CodeGenerator::setPreformatting(WrapMode lineWrappingStyle,
+                                     unsigned int lineLength,
+                                     int numberSpaces ){
+  bool enableWrap = lineWrappingStyle!=WRAP_DISABLED;
+  bool replaceTabs = numberSpaces > 0;
+  if (enableWrap || replaceTabs)  {
+    preFormatter=new PreFormatter(enableWrap, replaceTabs);
+    if (enableWrap)
+      preFormatter->setWrappingProperties(lineLength, lineWrappingStyle==WRAP_DEFAULT);
+    if (replaceTabs)
+      preFormatter->setNumberSpaces(numberSpaces);
   }
 }
 
+/*
 WrapMode CodeGenerator::getLineWrapping(){
-  if (lineWrapper==NULL) return WRAP_DISABLED;
-  return (lineWrapper->indentCode()?WRAP_DEFAULT:WRAP_SIMPLE);
+  if (preFormatter==NULL) return WRAP_DISABLED;
+  return (preFormatter->indentCode()?WRAP_DEFAULT:WRAP_SIMPLE);
 }
-
+*/
 LanguageDefinition &CodeGenerator::getLanguage(){
   return langInfo;
 }
@@ -213,7 +215,6 @@ LanguageDefinition &CodeGenerator::getLanguage(){
 void CodeGenerator::reset()
 {
   lineIndex = lineNumber = 0;
-  //showLineNumbers = lineNumberFillZeroes = fragmentOutput = false;
   line.clear();
 }
 
@@ -308,12 +309,12 @@ unsigned char CodeGenerator::getInputChar()
 
   if (eol) {
     bool eof=false;
-    if (lineWrapper!=NULL){
-       if (!lineWrapper->hasMoreLines()) {
+    if (preFormatter!=NULL){
+       if (!preFormatter->hasMoreLines()) {
           eof=readNewLine(line);
-          lineWrapper->setLine(line);
+          preFormatter->setLine(line);
        }
-       line = lineWrapper->getNextLine();
+       line = preFormatter->getNextLine();
     } else {
       eof=readNewLine(line);
     }
@@ -343,20 +344,15 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
   if (c=='\0') {
     return _EOF;   // End of file
   }
-  // replace tabs by spaces
-  if (isspace(c))
-    {
-      if (c=='\t'){
-         if (numberSpaces){
-            for (int space=0; space < numberSpaces; space++){
-              token += spacer;
-            }
-         } else {
-           token=c;
-         }
-       } else {
-          token=spacer;
-       }
+
+  if (isspace(c)) {
+        // keep original tabs if not replaced with -t option
+        /*if (c=='\t'){
+          token= c;
+        } else {
+          token= spacer;
+        }*/
+      token= c;
       return _WS;
     }
 
@@ -433,7 +429,7 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
   }
 
   // Alphanumerisches Token parsen und als Keyword oder Type erkennen
-  if (StringTools::isAlpha(c) || langInfo.isPrefix(c))
+  if (StringTools::isAlpha(c) || langInfo.isPrefix(c) || isAllowedChar(c))
     {
       if (langInfo.isPrefix(c)){
         token = c;
@@ -444,11 +440,20 @@ State CodeGenerator::getCurrentState (bool lastStateWasNumber)
       }
       string reservedWord=(langInfo.isIgnoreCase()) ?
                           StringTools::lowerCase(token):token;
+      /*int newKeywordClass=langInfo.isKeyword(reservedWord);
+      if (newKeywordClass){
+          currentKeywordClass = newKeywordClass;
+          return KEYWORD;
+      } else {
+          return STANDARD;
+      }*/
+
       currentKeywordClass=langInfo.isKeyword(reservedWord);
       return (currentKeywordClass) ? KEYWORD : STANDARD;
+
     }
 
-  // Character wurde nicht erkannt, in Standard-Zustand gehen
+  // Character not referring to any state
   token = c;
   return STANDARD;
 }
@@ -471,8 +476,7 @@ void CodeGenerator::printMaskedToken(bool flushWhiteSpace)
 
 bool CodeGenerator::isAllowedChar(char c)
 {
-  string allowedChars=langInfo.getAllowedChars();
-  return ( allowedChars.find(c)!= string::npos);
+  return ( langInfo.getAllowedChars().find(c)!= string::npos);
 }
 
 bool CodeGenerator::styleFound(){
@@ -676,7 +680,7 @@ LoadResult CodeGenerator::initLanguage(const string& langDefPath){
 ParseError CodeGenerator::printOutput (const string & inFileName,
                                        const string &outFileName)
 {
-  if (!docStyle.found()){
+  if (!docStyle.found()) {
     return BAD_STYLE;
   }
   reset();
@@ -686,7 +690,7 @@ ParseError CodeGenerator::printOutput (const string & inFileName,
   in = (inFileName.empty()? &cin :new ifstream (inFileName.c_str()));
   if (!in->fail()) {
     out = (outFileName.empty()? &cout :new ofstream (outFileName.c_str()));
-    if ( out->fail()){
+    if ( out->fail()) {
       error=BAD_OUTPUT;
     }
   }
@@ -718,24 +722,37 @@ ParseError CodeGenerator::printOutput (const string & inFileName,
   return error;
 }
 
-unsigned int CodeGenerator::getStyleID(State s){
-  if (s==KEYWORD){
-    return NUMBER_BUILTIN_STYLES + currentKeywordClass-1;
+
+unsigned int CodeGenerator::getStyleID(State s, unsigned int kwClassID){
+    if (s==KEYWORD && kwClassID){
+      return NUMBER_BUILTIN_STYLES + kwClassID-1;
   }
   return (unsigned int) s ;
 }
 
 void CodeGenerator::closeTag(State s){
-  *out << styleTagClose[getStyleID(s)];
+  *out << styleTagClose[(unsigned int)s];
   flushWs();
   currentState=_UNKNOWN;
 }
 
 void CodeGenerator::openTag(State s){
-  *out << styleTagOpen[getStyleID(s)];
+  *out << styleTagOpen[(unsigned int)s];
   currentState=s;
 }
 
+void CodeGenerator::closeKWTag(unsigned int kwClassID){
+    *out << styleTagClose[getStyleID(KEYWORD, kwClassID)];
+    flushWs();
+    currentState=_UNKNOWN;
+}
+
+void CodeGenerator::openKWTag(unsigned int kwClassID){
+    *out << styleTagOpen[getStyleID(KEYWORD, kwClassID)];
+    currentState=KEYWORD;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 void CodeGenerator::processRootState()
@@ -833,7 +850,8 @@ bool CodeGenerator::processKeywordState(State myState){
   bool eof=false,
        exitState=false;
 
-  openTag(KEYWORD);
+  //openTag(KEYWORD);
+   openKWTag(myClassID);
   do {
     printMaskedToken(newState!=_WS);
     newState= getCurrentState();
@@ -861,7 +879,9 @@ bool CodeGenerator::processKeywordState(State myState){
       }
   } while ((!exitState) && (!eof));
 
-  closeTag(KEYWORD);
+  //closeTag(KEYWORD);
+  closeKWTag(myClassID);
+
   currentKeywordClass=0;
   return eof;
 }
@@ -1191,7 +1211,7 @@ bool CodeGenerator::processEscapeCharState()
         break;
       case _WS:
         processWsState();
-        lineIndex--;
+        --lineIndex;
         break;
       case _EOF:
         eof = true;
@@ -1236,31 +1256,29 @@ void CodeGenerator::processWsState()
     return;
   }
 
-  // TODO : Move block to start of method;
-  // avoid escaping of single whitespace in LaTeX/TeX
-  int wscnt=0;
+  int cntWs=0;
   lineIndex--;
-  while (isspace(line[lineIndex])) {
-    wscnt+=(line[lineIndex]==' ') ? 1 : numberSpaces;
+
+   while (isspace(line[lineIndex])  ) {
+    ++cntWs;
     ++lineIndex;
   }
-  // TODO end
 
-  unsigned int styleID=getStyleID(currentState);
-  if (excludeWs && styleID!=_UNKNOWN) {
-     *out << styleTagClose[styleID];
-  }
-  if (wscnt>1){
+  if (cntWs>1) {
+    unsigned int styleID=getStyleID(currentState, currentKeywordClass);
+    if (excludeWs && styleID!=_UNKNOWN) {
+        *out << styleTagClose[styleID];
+    }
     *out << maskWsBegin;
-  }
-  for (int i=0;i<wscnt;i++){
-    *out << spacer;
-  }
-  if (wscnt>1){
+    for (int i=0; i<cntWs; i++){
+        *out << spacer;
+    }
     *out << maskWsEnd;
-  }
-  if (excludeWs && styleID!=_UNKNOWN){
-     *out << styleTagOpen[styleID];
+    if (excludeWs && styleID!=_UNKNOWN){
+        *out << styleTagOpen[styleID];
+    }
+  } else {
+    *out << token;
   }
   token.clear();
 }
@@ -1295,7 +1313,7 @@ void CodeGenerator::insertLineNumber(bool insertNewLine) {
     }
     os <<setw(LINE_NUMBER_WIDTH) << right << lineNumber;
 
-    unsigned int styleID=getStyleID(currentState);
+    unsigned int styleID=getStyleID(currentState, currentKeywordClass);
 
     if (styleID!=_UNKNOWN){
       *out << styleTagClose[styleID];
