@@ -32,7 +32,7 @@ void HighlightApp::printVersionInfo()
        #ifdef USE_LOCAL_GETOPT
          << "\n\n (Built with USE_LOCAL_GETOPT flag.)\n"
        #endif
-       << "\n Copyright (C) 2002-2004 Andre Simon <andre.simon1@gmx.de>"
+       << "\n Copyright (C) 2002-2005 Andre Simon <andre.simon1@gmx.de>"
        << "\n\n Artistic Style Classes (1.15.3)"
        << "\n Copyright (C) 1998-2002 Tal Davidson <davidsont@bigfoot.com>"
        << "\n\n Dirstream Classes (0.4)"
@@ -209,11 +209,6 @@ int HighlightApp::run(int argc, char**argv){
 
   // set data directory path, where /langDefs and /themes reside
   string highlightRootDir = Platform::getAppPath();
-  /*#ifdef _WIN32
-    highlightRootDir = argv[0];
-    string::size_type pos = highlightRootDir.find_last_of(Platform::pathSeparator);
-    highlightRootDir = highlightRootDir.substr(0, pos+1);
-  #endif*/
 
   // determine highlight data directory
   if (! dataDir.searchDataDir((options.dataDirGiven())?
